==> harmonizer-0.0.4.tgz: package/transform/patterns.js <==
'use strict';
var objectPatternC = require('nodes');
var nodes = objectPatternC.nodes;
var syntax = require('nodes/syntax.json');
var objectPatternB = require('../util/string');
var express = objectPatternB.express, lower = objectPatternB.lower, upper = objectPatternB.upper;
var objectPatternA = require('../util/insertion');
var insertBefore = objectPatternA.insertBefore;
var objectPattern = require('../util/id');
var getUniqueId = objectPattern.getUniqueId;
var createDeclarator = function (id, init) {
  return new nodes.VariableDeclarator({
    id: id,
    init: init
  });
};
var createAssignment = function (left, right) {
  return new nodes.AssignmentExpression({
    operator: '=',
    left: left,
    right: right
  });
};
var destruct = {
    ArrayPattern: function (pattern, declarations, valueId, assign) {
      var create = assign ? createAssignment : createDeclarator;
      pattern.elements.forEachRight(function (element, i) {
        if (element == null) {
          return;
        }
        var memberString = valueId ? valueId.name + '[' + i + ']' : null;
        if (element.type === syntax.Identifier) {
          declarations.unshift(create(element, memberString ? express(memberString).expression : null));
        } else {
          if (element.search('properties > * > value#Identifier, elements > #Identifier').length) {
            var nestedId;
            if (valueId) {
              nestedId = getUniqueId(declarations.scope(), valueId.name + i);
              var declaration = express('var ' + nestedId.name + ' = ' + memberString);
              insertBefore(declarations, declaration);
            }
            destruct[element.type](element, declarations, nestedId, assign);
          }
        }
      });
    },
    ObjectPattern: function (pattern, declarations, valueId, assign) {
      var create = assign ? createAssignment : createDeclarator;
      pattern.properties.forEachRight(function (property) {
        var memberString = valueId ? valueId.name + '.' + property.key.name : null;
        var value = property.value;
        if (value.type === syntax.Identifier) {
          declarations.unshift(create(value, memberString ? express(memberString).expression : null));
        } else {
          if (value.search('properties > * > value#Identifier, elements > #Identifier').length) {
            var nestedId;
            if (valueId) {
              nestedId = getUniqueId(declarations.scope(), valueId.name + upper(property.key.name));
              var declaration = express('var ' + nestedId.name + ' = ' + memberString);
              insertBefore(declarations, declaration);
            }
            destruct[value.type](value, declarations, nestedId, assign);
          }
        }
      });
    }
  };
function patternify(program) {
  var q;
  q = [
    '#ForOfStatement > left > declarations > * > #ArrayPattern',
    '#ForOfStatement > left > declarations > * > #ObjectPattern',
    '#ForInStatement > left > declarations > * > #ArrayPattern',
    '#ForInStatement > left > declarations > * > #ObjectPattern'
  ];
  program.search(q).forEachRight(function (pattern) {
    var declarator = pattern.parentNode;
    var declarations = declarator.parentNode;
    var declaration = declarations.parentNode;
    var forStatement = declaration.parentNode;
    var valueId = getUniqueId(forStatement.scope(), lower(pattern.type));
    declarations.replaceChild(declarator, new nodes.VariableDeclarator({
      id: valueId,
      kind: declaration.kind
    }));
    var newDeclaration = new nodes.VariableDeclaration();
    forStatement.body.body.unshift(newDeclaration);
    destruct[pattern.type](pattern, newDeclaration.declarations, valueId);
  });
  q = [
    '#ForOfStatement > left#ArrayPattern',
    '#ForOfStatement > left#ObjectPattern',
    '#ForInStatement > left#ArrayPattern',
    '#ForInStatement > left#ObjectPattern'
  ];
  program.search(q).forEachRight(function (pattern) {
    var forStatement = pattern.parentNode;
    var valueId = getUniqueId(forStatement.scope(), lower(pattern.type));
    forStatement.left = express('var ' + valueId.name);
    var expression = new nodes.ExpressionStatement();
    var sequence = new nodes.SequenceExpression();
    expression.expression = sequence;
    forStatement.body.body.unshift(expression);
    destruct[pattern.type](pattern, sequence.expressions, valueId, true);
  });
  q = '#VariableDeclarator > #ArrayPattern, #VariableDeclarator > #ObjectPattern';
  program.search(q).forEachRight(function (pattern) {
    var declarator = pattern.parentNode;
    var declarations = declarator.parentNode;
    var declaration = declarations.parentNode;
    declarations.removeChild(declarator);
    if (!pattern.search('properties > * > value#Identifier, elements > #Identifier').length) {
      if (!declarations.length) {
        declaration.parentNode.removeChild(declaration);
      }
      return;
    }
    if (declarator.init == null) {
      destruct[pattern.type](pattern, declarations);
    } else {
      var valueId;
      if (declarator.init.type === syntax.Identifier) {
        valueId = declarator.init;
      } else {
        valueId = getUniqueId(declarations.scope(), lower(pattern.type));
        var valueDeclaration = express('var ' + valueId.name + ' = $');
        valueDeclaration.declarations[0].init = declarator.init;
        insertBefore(declarations, valueDeclaration);
      }
      destruct[pattern.type](pattern, declarations, valueId);
    }
  });
  q = '#AssignmentExpression > left#ArrayPattern, #AssignmentExpression > left#ObjectPattern';
  program.search(q).forEachRight(function (pattern) {
    var expression = pattern.parentNode;
    var right = expression.right;
    var expressions = expression.parentNode;
    var sequence = expressions.parentNode;
    if (sequence.type !== syntax.SequenceExpression) {
      var key = expressions.indexOf(expression);
      expressions[key] = new nodes.SequenceExpression({ expressions: [expressions[key]] });
      expressions = expressions[key].expressions;
    }
    expressions.removeChild(expression);
    sequence = expressions.parentNode;
    var valueId;
    if (right.type === syntax.Identifier) {
      valueId = right;
    } else {
      valueId = getUniqueId(sequence.scope(), lower(pattern.type));
      var declaration = express('var ' + valueId.name + ' = $');
      declaration.declarations[0].init = right;
      insertBefore(sequence, declaration);
    }
    destruct[pattern.type](pattern, expressions, valueId, true);
  });
  q = '#Function > params > #ArrayPattern, #Function > params > #ObjectPattern';
  program.search(q).forEachRight(function (pattern) {
    var params = pattern.parentNode;
    var fn = params.parentNode;
    var valueId = getUniqueId(fn, lower(pattern.type));
    params.replaceChild(pattern, valueId);
    if (!pattern.search('properties > * > value#Identifier, elements > #Identifier').length) {
      return;
    }
    var declaration = new nodes.VariableDeclaration();
    fn.body.body.unshift(declaration);
    destruct[pattern.type](pattern, declaration.declarations, valueId);
  });
}
exports.transform = patternify;

==> harmonizer-0.2.0.tgz: package/transform/patterns.js <==
'use strict';
var callExpressionC = require('nodes');
var nodes = callExpressionC.nodes, syntax = callExpressionC.syntax;
var callExpressionB = require('../util/string');
var express = callExpressionB.express, lower = callExpressionB.lower, upper = callExpressionB.upper;
var callExpressionA = require('../util/insertion');
var insertBefore = callExpressionA.insertBefore;
var callExpression = require('../util/id');
var getUniqueId = callExpression.getUniqueId;
var createDeclarator = function (id, init) {
  return new nodes.VariableDeclarator({
    id: id,
    init: init
  });
};
var createAssignment = function (left, right) {
  return new nodes.AssignmentExpression({
    operator: '=',
    left: left,
    right: right
  });
};
var destruct = {
    ArrayPattern: function (pattern, declarations, valueId, assign) {
      var create = assign ? createAssignment : createDeclarator;
      pattern.elements.forEachRight(function (element, i) {
        if (element == null) {
          return;
        }
        var memberString = valueId ? valueId.name + '[' + i + ']' : null;
        if (element.type === syntax.Identifier) {
          declarations.unshift(create(element, memberString ? express(memberString).expression : null));
        } else {
          if (element.search('properties > * > value#Identifier, elements > #Identifier').length) {
            var nestedId;
            if (valueId) {
              nestedId = getUniqueId(declarations.scope(), valueId.name + i);
              var declaration = express('var ' + nestedId.name + ' = ' + memberString);
              insertBefore(declarations, declaration);
            }
            destruct[element.type](element, declarations, nestedId, assign);
          }
        }
      });
    },
    ObjectPattern: function (pattern, declarations, valueId, assign) {
      var create = assign ? createAssignment : createDeclarator;
      var properties = pattern.properties;
      properties.forEachRight(function (property) {
        var memberString = valueId ? valueId.name + '.' + property.key.name : null;
        var value = property.value;
        if (value.type === syntax.Identifier) {
          declarations.unshift(create(value, memberString ? express(memberString).expression : null));
        } else {
          if (value.search('properties > * > value#Identifier, elements > #Identifier').length) {
            var nestedId;
            if (valueId) {
              nestedId = getUniqueId(declarations.scope(), valueId.name + upper(property.key.name));
              var declaration = express('var ' + nestedId.name + ' = ' + memberString);
              insertBefore(declarations, declaration);
            }
            destruct[value.type](value, declarations, nestedId, assign);
          }
        }
      });
    }
  };
exports.default = function patternify(program) {
  var q;
  q = [
    '#ForOfStatement > left > declarations > * > #ArrayPattern',
    '#ForOfStatement > left > declarations > * > #ObjectPattern',
    '#ForInStatement > left > declarations > * > #ArrayPattern',
    '#ForInStatement > left > declarations > * > #ObjectPattern'
  ];
  program.search(q).forEachRight(function (pattern) {
    var declarator = pattern.parentNode;
    var declarations = declarator.parentNode;
    var declaration = declarations.parentNode;
    var forStatement = declaration.parentNode;
    var valueId = getUniqueId(forStatement.scope(), lower(pattern.type));
    declarations.replaceChild(declarator, new nodes.VariableDeclarator({
      id: valueId,
      kind: declaration.kind
    }));
    var newDeclaration = new nodes.VariableDeclaration();
    forStatement.body.body.unshift(newDeclaration);
    destruct[pattern.type](pattern, newDeclaration.declarations, valueId);
  });
  q = [
    '#ForOfStatement > left#ArrayPattern',
    '#ForOfStatement > left#ObjectPattern',
    '#ForInStatement > left#ArrayPattern',
    '#ForInStatement > left#ObjectPattern'
  ];
  program.search(q).forEachRight(function (pattern) {
    var forStatement = pattern.parentNode;
    var valueId = getUniqueId(forStatement.scope(), lower(pattern.type));
    forStatement.left = express('var ' + valueId.name);
    var expression = new nodes.ExpressionStatement();
    var sequence = new nodes.SequenceExpression();
    expression.expression = sequence;
    forStatement.body.body.unshift(expression);
    destruct[pattern.type](pattern, sequence.expressions, valueId, true);
  });
  q = '#VariableDeclarator > #ArrayPattern, #VariableDeclarator > #ObjectPattern';
  program.search(q).forEachRight(function (pattern) {
    var declarator = pattern.parentNode;
    var declarations = declarator.parentNode;
    var declaration = declarations.parentNode;
    declarations.removeChild(declarator);
    if (declarator.init == null) {
      destruct[pattern.type](pattern, declarations);
    } else {
      var valueId;
      if (declarator.init.type === syntax.Identifier) {
        valueId = declarator.init;
      } else {
        valueId = getUniqueId(declarations.scope(), lower(declarator.init.type));
        var valueDeclaration = express('var ' + valueId.name + ' = $');
        valueDeclaration.declarations[0].init = declarator.init;
        insertBefore(declarations, valueDeclaration);
      }
      destruct[pattern.type](pattern, declarations, valueId);
    }
    if (!declarations.length) {
      declaration.parentNode.removeChild(declaration);
    }
  });
  q = '#AssignmentExpression > left#ArrayPattern, #AssignmentExpression > left#ObjectPattern';
  program.search(q).forEachRight(function (pattern) {
    var expression = pattern.parentNode;
    var right = expression.right;
    var expressions = expression.parentNode;
    var sequence = expressions.parentNode;
    if (sequence.type !== syntax.SequenceExpression) {
      var key = expressions.indexOf(expression);
      expressions[key] = new nodes.SequenceExpression({ expressions: [expressions[key]] });
      expressions = expressions[key].expressions;
    }
    expressions.removeChild(expression);
    sequence = expressions.parentNode;
    var valueId;
    if (right.type === syntax.Identifier) {
      valueId = right;
    } else {
      valueId = getUniqueId(sequence.scope(), lower(right.type));
      var declaration = express('var ' + valueId.name + ' = $');
      declaration.declarations[0].init = right;
      insertBefore(sequence, declaration);
    }
    destruct[pattern.type](pattern, expressions, valueId, true);
  });
  q = '#Function > params > #ArrayPattern, #Function > params > #ObjectPattern';
  program.search(q).forEachRight(function (pattern) {
    var params = pattern.parentNode;
    var fn = params.parentNode;
    var valueId = getUniqueId(fn, 'argument');
    params.replaceChild(pattern, valueId);
    if (!pattern.search('properties > * > value#Identifier, elements > #Identifier').length) {
      return;
    }
    var declaration = new nodes.VariableDeclaration();
    fn.body.body.unshift(declaration);
    destruct[pattern.type](pattern, declaration.declarations, valueId);
  });
};
